// Copyright (c) 2009-2015 Intel Corporation
// All rights reserved.
//
// WARRANTY DISCLAIMER
//
// THESE MATERIALS ARE PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
// "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
// LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
// A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL INTEL OR ITS
// CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
// EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
// PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
// PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
// OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY OR TORT (INCLUDING
// NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THESE
// MATERIALS, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//
// Intel Corporation is the author of the Materials, and requests that all
// problem reports or change requests be submitted to it directly


#define LOCAL_SIZE 4

//These are the OLD OpenCL 1.2 functions
void LocalXToY_internal(__local unsigned char *in, __global unsigned char *out, unsigned int startFrom, unsigned int startTo, unsigned int length);
void GlobalXTo_internal(__global unsigned char *in, __global unsigned char *out, unsigned int startFrom, unsigned int startTo, unsigned int length);
//replaced by this single OpenCL 2.0 function
void GenericXToY_internal(unsigned char *in, unsigned char *out, unsigned int startFrom, unsigned int startTo, unsigned int length);

__kernel void LocalXToY_OpenCL_2_0(__global unsigned char *in, __global unsigned char *out)
{
	unsigned int length = LOCAL_SIZE;
	__local unsigned char localIn[LOCAL_SIZE];
	unsigned int j=0;

	//copy a portion of X into local memory
	int gid = get_global_id(0);
	if(get_local_id(0) == 0)
	{
		GenericXToY_internal(in, localIn, gid, 0, length);

		//because you are using LOCAL memory always starts at index 0
		//using global memory for output so use gid
		GenericXToY_internal(localIn, out, 0, gid, length);
	}
	else
	{ 
		//no work for these workitems
	}
}

__kernel void GlobalXToY_OpenCL2_0(__global unsigned char *in, __global unsigned char *out, unsigned int startFrom, unsigned int startTo, unsigned int length)
{
	int gid = get_global_id(0);

	GenericXToY_internal(in, out, gid, gid, 1);

}

void GenericXToY_internal(unsigned char *in, unsigned char *out, unsigned int startFrom, unsigned int startTo, unsigned int length)
{
	for(int i = startFrom; i < startFrom + length;)
	{
		out[startTo++] = in[i++];
	}
}

/////////////////////////////////////////////////////////////////
// These are the functions that the Generic function is replacing
/////////////////////////////////////////////////////////////////

void LocalXToY_internal(__local unsigned char *in, __global unsigned char *out, unsigned int startFrom, unsigned int startTo, unsigned int length)
{
	for(int i = startFrom; i < startFrom + length;)
	{
		out[startTo++] = in[i++];
	}
}

void GlobalXToY_internal(__global unsigned char *in, __global unsigned char *out, unsigned int startFrom, unsigned int startTo, unsigned int length)
{	
	for(int i = startFrom; i < startFrom + length;)
	{
		out[startTo++] = in[i++];
	}
}